/*
 * gawk.r - Resources for projector & version identification.
 */

/* 
 * Copyright (C) 1986, 1988, 1989, 1991 the Free Software Foundation, Inc.
 * 
 * This file is part of GAWK, the GNU implementation of the
 * AWK Progamming Language.
 * 
 * GAWK is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 1, or (at your option)
 * any later version.
 * 
 * GAWK is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with GAWK; see the file COPYING.  If not, write to
 * the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "Types.r"
#include "SysTypes.r"
#include "Cmdo.r"

/**************************** Version identification ****************************/

resource 'vers' (1) {
	0x02, 0x11, release, 0x03, verGrSwiss,            	                                                                 	                        	            	         	                                                                    	                                                       /* Version #    */
	"2.11.1r3",                                             /* Short string */              
	"2.11.1r3"                      								  /* Long string  */
};

/**************************** Cursor images ****************************/

resource 'CURS' (128) {
	$"FFFE 8002 9E62 B092 BEF2 B692 9E92 8002"
	$"8002 A492 A4A2 A4C2 BCA2 A492 8002 FFFE",
	$"FFFE FFFE FFFE FFFE FFFE FFFE FFFE FFFE"
	$"FFFE FFFE FFFE FFFE FFFE FFFE FFFE FFFE",
	{8, 8}
};

resource 'CURS' (129) {
	$"FFFE 8002 9C72 A0DA ACFA A4DA 9CDA 8002"
	$"8002 A492 A4A2 A4C2 BCA2 A492 8002 FFFE",
	$"FFFE FFFE FFFE FFFE FFFE FFFE FFFE FFFE"
	$"FFFE FFFE FFFE FFFE FFFE FFFE FFFE FFFE",
	{8, 8}
};

resource 'CURS' (130) {
	$"FFFE 8002 9C62 A092 ACF2 A492 9C92 8002"
	$"8002 B692 B6A2 B6C2 BEA2 B692 8002 FFFE",
	$"FFFE FFFE FFFE FFFE FFFE FFFE FFFE FFFE"
	$"FFFE FFFE FFFE FFFE FFFE FFFE FFFE FFFE",
	{8, 8}
};

resource 'CURS' (131) {
	$"FFFE 8002 9C62 A092 ACF2 A492 9C92 8002"
	$"8002 A4DA A4F2 A4E2 BCF2 A4DA 8002 FFFE",
	$"FFFE FFFE FFFE FFFE FFFE FFFE FFFE FFFE"
	$"FFFE FFFE FFFE FFFE FFFE FFFE FFFE FFFE",
	{8, 8}
};

resource 'acur' (0) {
	{	128,
		129,
		130,
		131,
	}
};

/**************************** Commando dialog ****************************/

resource 'cmdo' (128) {
	{
		295,			/* Height of dialog */
		"Gawk, the GNU implementation of the AWK Programming Language.",
		{
			notDependent {}, RegularEntry {
				"Field separator:",
				{167,  10, 183, 115},
				{167, 120, 183, 195},
				" ",
				keepCase,
				"-F",
				"Character or regular expression used to separate fields."
			},
			notDependent {}, NestedDialog {
				2,
				{85,  25, 105, 185},
				"More Options",
				"Variable definitions and GNU specific options."
			},
			Or {{-4}}, MultiFiles {
				"Program File(s)",
				"Select program files.  If no files are specified, Gawk expects the program"
				"to be given on the command line.",
				{55,  25, 75, 185},
				"Program files:",
				"-f",
				MultiInputFiles {
					{TEXT},
					".awk",
					"Files ending with .awk",
					"All text files",
				}
			},
			Or {{-3}}, RegularEntry {
				"Program:",
				{142,  10, 158, 115},
				{142, 120, 158, 465},
				"",
				keepCase,
				"",
				"Program to execute."
			},
			Or {{-6}}, MultiFiles {
				"Input File(s)",
				"Select input files.  If no files are specified, Gawk reads from standard input.",
				{25, 25, 45, 185},
				"Input files:",
				"",
				MultiInputFiles {
					{TEXT},
					FilterTypes,
					"Only text files",
					"All files",
				}
			},
			Or {{-5}}, Redirection {
				StandardInput,
				{ 25, 215}
			},
			notDependent {}, Redirection {
				StandardOutput,
				{ 60, 215}
			},
			notDependent {}, Redirection {
				DiagnosticOutput,
				{ 95, 215}
			},
			notDependent {}, TextBox {
				gray,
				{ 20, 200, 135, 468},
				"Redirection"
			},
			notDependent {}, VersionDialog {
				VersionString {
					"2.11.1r3"
				},
				"Gawk for MPW. Gawk by Free Software Foundation, Inc.\n"
				"MPW port by Matthias Neeracher <neeri@iis.ethz.ch>.",
				0
			},
		},
		275,
		"Variable definitions and GNU specific options.",
		{
			notDependent {}, RadioButtons {
				{
					{ 35,  20,  51, 240},
					"Awk style",
					"-a",
					Set,
					"Use awk style regular expressions, as described in the book.",
					
					{ 55,  20,  71, 240},
					"Egrep style",
					"-e",
					NotSet,
					"Use egrep style regular expressions, as described in the POSIX standard.",
					
					{ 75,  20,  91, 240},
					"MPW style",
					"-m",
					NotSet,
					"Use MPW shell patterns."
				}
			},
			notDependent {}, TextBox {
				gray,
				{ 25,  15,  95, 245},
				"Regular expressions"
			},
			notDependent {}, CheckOption {
				NotSet,
				{105,  20, 121, 240},
				"Compatibility mode",
				"-c",
				"Behave exactly like UNIX awk, don't recognize any GNU specific extensions."
			},
			Or {{-3}}, CheckOption {
				NotSet,
				{130,  20, 146, 240},
				"Print copyright information",
				"-C",
				"Print short version of GNU copyright information message to standard output."
			},
			Or {{-3}}, CheckOption {
				NotSet,
				{150,  20, 166, 240},
				"Print version information",
				"-V",
				"Print version information for this version of gawk."
			},
			notDependent {}, MultiRegularEntry {
				"Variable Definitions",
				{ 10, 255,  26, 465},
				{ 28, 255, 161, 465},
				{},
				"-v",
				"var=val, assign the value val to the variable name var."
			}
		}
	}
};
